/*
 * Copyright (c) 2014-2019 Meltytech, LLC
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
// KEYFRAMES to b added.

import QtQuick 2.1
import QtQuick.Controls 1.0
import QtQuick.Layouts 1.0
import Shotcut.Controls 1.0
 
//import QtQuick 2.1
//import QtQuick.Controls 1.1
//import QtQuick.Controls.Styles 1.1
//import QtQuick.Layouts  1.0
import QtQuick.Dialogs  1.0
//import Shotcut.Controls 1.0

Item {

/*
 uniform vec2  Scope_Coords;   //= vec2(0.5, 0.5);          // KF - Normalised coordinates of scope
 uniform float Scope_Diameter; //= 0.333;                   // KF - Normalises diameter of scope (1.0 = video height)
 uniform float Mag_Index;      //= 1.0;						// KF - Degree of magnification (0.0 to 1.0)
 uniform float Scope_Thickness;//= 0.04;					// Scope thickness fraction of scope diameter)
 uniform vec4  Scope_Color;    //= vec4(0.1, 0.1, 0.2, 1,0);// Colour of the scope (normalised RGBA)
 uniform bool  Scope_Only;     //= false; 				    // true = only image within scope is shown
 uniform bool  Cross_Hairs;    //= true;					// true = scope cross-hairs are shown
*/

	property real   def_x_pos			: 0.5
	property real   def_y_pos			: 0.5
	property real   def_diameter		: 0.333
	property real   def_magnification	: 1.0
	property real   def_thickness		: 0.04
	property string def_color			: '#181830'
	property bool   def_cross_hairs		: true
	property bool   def_scope_only		: false

    property bool	blockUpdate			: true
    property var	startValues			: [0.25, 0.25, 0.333, 1.0] // KeyFrames: x_pos, y_pos, diameter, magnification
    property var	middleValues		: [0.5 , 0.5 , 0.444, 0.5]
    property var	endValues			: [0.75, 0.75, 0.555, 0.0]
	
    width : 350
    height: 225
	
    Component.onCompleted: {
        if (filter.isNew) {
//	        filter.set('resource'     , filter.path + 'debug.html')
            filter.set('resource'     , filter.path + 'Sniper_Scope.html')
            filter.set('x_pos'        , def_x_pos        )
            filter.set('y_pos'        , def_y_pos        )
			filter.set('diameter'     , def_diameter     )
            filter.set('magnification', def_magnification)
			filter.set('thickness'    , def_thickness    )
            filter.set('color'        , def_color        )
			filter.set('cross_hairs'  , def_cross_hairs  )
			filter.set('scope_only'   , def_scope_only   )
			
            filter.savePreset(preset.parameters)
		} else {
            initSimpleAnimation()
		}
        setControls();
		setKeyframedControls();
        filter.set('in' , producer.in)
        filter.set('out', producer.out)
    }
	
	function initSimpleAnimation() {
		middleValues 	= [filter.getDouble('x_pos'        , filter.animateIn),
						   filter.getDouble('y_pos'        , filter.animateIn),
                           filter.getDouble('diameter'     , filter.animateIn),
						   filter.getDouble('magnification', filter.animateIn)]
        if (filter.animateIn > 0) {
            startValues = [filter.getDouble('x_pos'        , 0),
                           filter.getDouble('y_pos'        , 0),
                           filter.getDouble('diameter'     , 0),
						   filter.getDouble('magnification', 0)]
        }
        if (filter.animateOut > 0) {
            endValues 	= [filter.getDouble('x_pos'        , filter.duration - 1),
                           filter.getDouble('y_pos'        , filter.duration - 1),
                           filter.getDouble('diameter'     , filter.duration - 1),
						   filter.getDouble('magnification', filter.duration - 1)]
        }
    }

    function getPosition() {
        return Math.max(producer.position - (filter.in - producer.in), 0)
    }

    function setKeyframedControls() {
        var position              = getPosition()
        blockUpdate               = true
                x_posSlider.value = filter.getDouble('x_pos'   , position) * 100
                y_posSlider.value = filter.getDouble('y_pos'   , position) * 100
             diameterSlider.value = filter.getDouble('diameter', position) * 100
        magnificationSlider.value = filter.getDouble('magnification', position)
        blockUpdate               = false
        x_posSlider.enabled       = y_posSlider.enabled  = diameterSlider.enabled = magnificationSlider.enabled
                                  =  position <= 0                      ||
							        (position >= (filter.animateIn - 1) &&
							         position <= (filter.duration - filter.animateOut)
							        )                                   ||
							         position >= (filter.duration - 1)
    }	
	
    function setControls() {
        var position = getPosition()
                x_posSlider.value   = filter.getDouble('x_pos'        ) * 100
                y_posSlider.value   = filter.getDouble('y_pos'        ) * 100
             diameterSlider.value   = filter.getDouble('diameter'     ) * 100
        magnificationSlider.value   = filter.getDouble('magnification')
            thicknessSlider.value   = filter.getDouble('thickness'    ) * 100
                colorSwatch.value   = filter.get      ('color'        )
        cross_hairsCheckBox.checked = filter.getDouble('cross_hairs'  )
         scope_onlyCheckBox.checked = filter.getDouble('scope_only'   )
    }

    function updateFilter(parameter, value, position, button) {
        if (blockUpdate) return
        var index = preset.parameters.indexOf(parameter)
        if (position !== null) {
            if (position <= 0 && filter.animateIn > 0)
                startValues[index]	= value
            else if (position >= filter.duration - 1 && filter.animateOut > 0)
                endValues[index]	= value
            else
                middleValues[index]	= value
        }

        if (filter.animateIn > 0 || filter.animateOut > 0) {
            filter.resetProperty(parameter)
           button.checked = false
            if (filter.animateIn > 0) {
                filter.set(parameter, startValues[index], 0)
                filter.set(parameter, middleValues[index], filter.animateIn - 1)
			}
            if (filter.animateOut > 0) {
                filter.set(parameter, middleValues[index], filter.duration - filter.animateOut)
                filter.set(parameter, endValues[index], filter.duration - 1)
            }
        } else if (!button.checked) {
            filter.resetProperty(parameter)
            filter.set			(parameter, middleValues[index])
        } else if (position !== null) {
            filter.set			(parameter, value, position)
        }
   }

    function onKeyframesButtonClicked(checked, parameter, value) {
        if (checked) {
            blockUpdate = true
            x_posSlider.enabled = y_posSlider.enabled = diameterSlider.enabled = true
            if (filter.animateIn > 0 || filter.animateOut > 0) {
                filter.resetProperty('x_pos')
                filter.resetProperty('y_pos')
                filter.resetProperty('diameter')
                filter.resetProperty('magnification')
				
                filter.animateIn = filter.animateOut = 0
            } else {
                filter.clearSimpleAnimation(parameter)
            }
            blockUpdate = false
            filter.set(parameter, value, getPosition())
        } else {
            filter.resetProperty(parameter)
            filter.set(parameter, value)
        }
    }

    GridLayout {
        columns			: 4
        anchors.fill	: parent
        anchors.margins	: 8

        Label {
            text: qsTr('Preset')
            Layout.alignment: Qt.AlignRight
        }

       Preset {
            id					: preset
            Layout.columnSpan	: parent.columns - 1
            parameters			: [ 'x_pos'        ,
									'y_pos'        ,
									'diameter'     ,
									'magnification',
									'thickness'    ,
									'color'        ,
									'cross_hairs'  ,
									'scope_only'
								  ]						// ['x_pos', 'y_pos' .....]
            onPresetSelected	: {
									setControls()
									setKeyframedControls()
									initSimpleAnimation()
			}
        }

        Label {
            text				: qsTr('X-pos')
            Layout.alignment	: Qt.AlignRight
        }
        SliderSpinner {
            id					: x_posSlider
			minimumValue		: 0
            maximumValue		: 100
			stepSize			: 0.05
			decimals			: 2
            suffix				: ' %'
            value				: filter.getDouble('x_pos') * 100
            onValueChanged		: updateFilter('x_pos', value / 100, getPosition(), x_posKeyframesButton)

        }
        UndoButton {
            onClicked			: x_posSlider.value = def_x_pos * 100
        }
        KeyframesButton {
            id					: x_posKeyframesButton
            checked				: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('x_pos') > 0
            onToggled			: onKeyframesButtonClicked(checked, 'x_pos', x_posSlider.value / 100)
        }

        Label {
            text				: qsTr('Y-pos')
            Layout.alignment	: Qt.AlignRight
        }
        SliderSpinner {
            id					: y_posSlider
            minimumValue		: 0
            maximumValue		: 100
			stepSize			: 0.05
			decimals			: 2
            suffix				: ' %'
            value				: filter.getDouble('y_pos') * 100
            onValueChanged		: updateFilter('y_pos', value / 100, getPosition(), y_posKeyframesButton)

        }
        UndoButton {
            onClicked			: y_posSlider.value = def_y_pos * 100
        }
        KeyframesButton {
            id					: y_posKeyframesButton
            checked				: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('y_pos') > 0
            onToggled			: onKeyframesButtonClicked(checked, 'y_pos', y_posSlider.value / 100)
        }

        Label {
            text				: qsTr('Diameter')
            Layout.alignment	: Qt.AlignRight
        }
        SliderSpinner {
            id					: diameterSlider
            minimumValue		: 0
            maximumValue		: 100
 			stepSize			: 1.0
			decimals			: 0
            suffix				: ' %'
            value				: filter.getDouble('diameter') * 100
            onValueChanged		: updateFilter('diameter', value / 100, getPosition(), diameterKeyframesButton)

        }
        UndoButton {
            onClicked			: diameterSlider.value = def_diameter * 100
        }
        KeyframesButton {
            id					: diameterKeyframesButton
            checked				: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('diameter') > 0
            onToggled			: onKeyframesButtonClicked(checked, 'diameter', diameterSlider.value / 100)
        }

        Label {
            text				: qsTr('Magnification')
            Layout.alignment	: Qt.AlignRight
        }
        SliderSpinner {
            id					: magnificationSlider
            minimumValue		: 0
            maximumValue		: 1.0
 			stepSize			: 0.1
			decimals			: 1
            value				: filter.getDouble('magnification')
            onValueChanged		: updateFilter('magnification', value, getPosition(), magnificationKeyframesButton) 
        }
        UndoButton {
            onClicked			: magnificationSlider.value = def_magnification
        }
        KeyframesButton {
            id					: magnificationKeyframesButton
            checked				: filter.animateIn <= 0 && filter.animateOut <= 0 && filter.keyframeCount('magnification') > 0
            onToggled			: onKeyframesButtonClicked(checked, 'magnification', magnificationSlider.value )
        }

        Label {
            text				: qsTr('Thickness')
            Layout.alignment	: Qt.AlignRight
        }
        SliderSpinner {
            id					: thicknessSlider
            minimumValue		: 0
            maximumValue		: 25
 			stepSize			: 1.0
			decimals			: 0
            suffix				: ' %'
            value				: filter.getDouble('thickness') * 100
            onValueChanged		: filter.set('thickness', value / 100)
        }
        UndoButton {
            onClicked			:thicknessSlider.value = def_thickness * 100
        }
        Item { Layout.fillWidth	: true }

        Label {
            text					: qsTr('Color')
            Layout.alignment		: Qt.AlignRight
        }
        ColorPicker {
            id						: colorSwatch
            value					: filter.get('color')
            property bool isReady	: false
            Component.onCompleted	: isReady = true
            onValueChanged			: {
                if (isReady) {
                    filter.set('color'        , '' + value)
                    filter.set("disable"      , 0);
                }
            }
            onPickStarted			: filter.set("disable", 1)
            onPickCancelled			: filter.set('disable', 0)
        }
        UndoButton {
            onClicked				: colorSwatch.value = def_color
        }
        Item { Layout.fillWidth		: true }		
 
        Label {
            text					: qsTr('Options')
            Layout.alignment		: Qt.AlignRight
        }
		CheckBox {
            id						: cross_hairsCheckBox
            Layout.alignment		: Qt.AlignLeft
            checked					: filter.getDouble('cross_hairs')
            text					: qsTr('Cross-hairs')
            onCheckedChanged		: filter.set('cross_hairs', checked)
        }
        Label {
            text					: ' '
            Layout.alignment		: Qt.AlignLeft
        }
        Item { Layout.fillWidth		: true }		
 
        Label {
            text					: qsTr('Options')
            Layout.alignment		: Qt.AlignRight
        }
		CheckBox {
            id						: scope_onlyCheckBox
            Layout.alignment		: Qt.AlignLeft
            checked					: filter.getDouble('scope_only')
            text					: qsTr('Scope-only')
            onCheckedChanged		: filter.set('scope_only', checked)
        }
        Item { Layout.fillWidth		: true }
 //       Item { Layout.fillHeight	: true }
   }

    function updateSimpleAnimation() {
        updateFilter('x_pos'        ,         x_posSlider.value / 100, getPosition(),         x_posKeyframesButton)
        updateFilter('y_pos'        ,         y_posSlider.value / 100, getPosition(),         y_posKeyframesButton)
        updateFilter('diameter'     ,      diameterSlider.value / 100, getPosition(),      diameterKeyframesButton)
        updateFilter('magnification', magnificationSlider.value      , getPosition(), magnificationKeyframesButton)
	}

    Connections {
        target				: filter
        onInChanged			: updateSimpleAnimation()
        onOutChanged		: updateSimpleAnimation()
        onAnimateInChanged	: updateSimpleAnimation()
        onAnimateOutChanged	: updateSimpleAnimation()
    }

    Connections {
        target				: producer
        onPositionChanged	: setKeyframedControls()
    }
}
